
require(MPTinR)     # for fitting
require(snowfall)   # for multicore 
require(mail)       # for sending results to mail

# first generate all possible proportions (stepsize is by)
# then fit all possible porportions.
# then see if misfit exists.

by <- 0.005
to_integer <- 1000
by*to_integer
e <- by/100

n_optim <- 20
stepsize <- 1000

nCPU <- 4  # uses nCPU cores for multicore fitting

# results are automatically send to following mail address (cannot contain . or _, gmail.com works well)
mail_address <- "yourname@gmail.com"

# fitting is performed in steps (i.e., not all proportions at once) using the following function
# stepsize determines how many sets of probabilities are fitted at once.
fit.stepwise <- function(data, model, stepsize, multicore = "individual") {
  steps <- c(seq(1, nrow(data), by = stepsize), nrow(data))
  out <- matrix(-1, nrow=nrow(data), ncol=4)
  for (i in 2:length(steps)) {
    tmp_fit <- fit.model(data[steps[i-1]:steps[i],], textConnection(model), n.optim=n_optim, multicore=multicore, fit.aggregated=FALSE, use.gradient=FALSE, numDeriv=FALSE)
    out[steps[i-1]:steps[i],] <- abs(tmp_fit$data$observed$individual - tmp_fit$data$predicted$individual)
  }
  #browser()
  
  model_char <- as.character(match.call()[[3]])
  mis_in <- rowSums(out)  
  
  text_to_mail <- paste(model_char, "\n", "max possible sets (indiv):", max(out),"\n", "deviation (r/c =", paste(which(out == max(out), arr.ind=TRUE), collapse = " "), "):", paste(out[which(out == max(out), arr.ind=TRUE)[,1],], collapse = " "),"\n", "observed  (r/c =", paste(which(out == max(out), arr.ind=TRUE), collapse= " "), "):", paste(data[which(out == max(out), arr.ind=TRUE)[,1],], collapse = " "), "\n", "\n",
                        "max possible sets (summed per row):", max(mis_in),"\n", 
  "deviation (row =", which(mis_in == max(mis_in)), "):", paste(out[which(mis_in == max(mis_in)),], collapse=" "),"\n", 
  "observed  (row =", which(mis_in == max(mis_in)), "):", paste(data[which(mis_in == max(mis_in)),], collapse=" "), "\n", "\n", "\n")
  
  sendmail(mail_address, paste("model", model_char), text_to_mail)
  
  cat(model, "\n")
  cat("max possible sets (indiv):", max(out),"\n")
  cat("deviation (r/c =", which(out == max(out), arr.ind=TRUE), "):", out[which(out == max(out), arr.ind=TRUE)[,1],],"\n")
  cat("observed  (r/c =", which(out == max(out), arr.ind=TRUE), "):", data[which(out == max(out), arr.ind=TRUE)[,1],], "\n", "\n")
  
  
  cat("max possible sets (summed per row):", max(mis_in),"\n")
  cat("deviation (row =", which(mis_in == max(mis_in)), "):", out[which(mis_in == max(mis_in)),],"\n")
  cat("observed  (row =", which(mis_in == max(mis_in)), "):", data[which(mis_in == max(mis_in)),], "\n", "\n", "\n")
  
  out
  
}




## Pattern V:
# A > B > D and A > C > D

res <- matrix(-1, nrow=1e07, ncol = 4)

i <- 1
for (a in seq(0,1, by=by)) {
  for (b in seq(0, (1-a), by = by)) {
    for (c in seq(0,(1-a-b), by = by)) {
      d <- 1-sum(c(a, b, c))
      if ( (a > (b-e)) && (a > (c-e)) && (b > (d-e)) && (c > (d-e)) ) {
        res[i,] <- c(a, b, c, d)
        i  <- i+1
      }
    }
  }
}



res_p5 <- round(res[1:(i-1),] * to_integer)

head(res_p5, 20)
nrow(res_p5)

p5 <- "

(1-x1)*(1-x2)*(1-x3) + x1*(1-x2)*(1-x3)*(1/2) + (1-x1)*x2*(1-x3)*(1/2) + x1*x2*(1-x3)*(1/3) + x3*(1/4)
                       x1*(1-x2)*(1-x3)*(1/2) +                          x1*x2*(1-x3)*(1/3) + x3*(1/4)
                                                (1-x1)*x2*(1-x3)*(1/2) + x1*x2*(1-x3)*(1/3) + x3*(1/4)
x3*(1/4)
"

check.mpt(textConnection(p5))



sfInit( parallel=TRUE, cpus=nCPU, type = "SOCK" )

#deviances_p5 <- fit.stepwise(res_p5[1:100,], p5, 10, multicore="none")

deviances_p5 <- fit.stepwise(res_p5, p5, stepsize)
sfStop()

save(deviances_p5, file = "deviances_p5.rda")



## Pattern VI:
# A > B, B > C, and B > D
res <- matrix(-1, nrow=1e07, ncol = 4)

i <- 1
for (a in seq(0,1, by=by)) {
  for (b in seq(0, (1-a), by = by)) {
    for (c in seq(0,(1-a-b), by = by)) {
      d <- 1-sum(c(a, b, c))
      if ( (a > (b-e)) && (b > (c-e)) && (b > (d-e)) ) {
        res[i,] <- c(a, b, c, d)
        i  <- i+1
      }
    }
  }
}


res_p6 <- round(res[1:(i-1),] * to_integer)

head(res_p6, 20)

p6 <- "
x1 + (1-x1)*(1-x2)*(1-x3)*0.5 + (1-x1)*x2*(1-x3)*(1/3) + (1-x1)*(1-x2)*x3*(1/3) + (1-x1)*x2*x3*(1/4)
     (1-x1)*(1-x2)*(1-x3)*0.5 + (1-x1)*x2*(1-x3)*(1/3) + (1-x1)*(1-x2)*x3*(1/3) + (1-x1)*x2*x3*(1/4)
                                (1-x1)*x2*(1-x3)*(1/3) +                          (1-x1)*x2*x3*(1/4)
                                                         (1-x1)*(1-x2)*x3*(1/3) + (1-x1)*x2*x3*(1/4)
"

check.mpt(textConnection(p6))

sfInit( parallel=TRUE, cpus=nCPU, type = "SOCK" )
deviances_p6 <- fit.stepwise(res_p6, p6, stepsize)
sfStop()

save(deviances_p6, file = "deviances_p6.rda")


## Pattern VIII:
# A > C, B > C, and B > D

res <- matrix(-1, nrow=1e07, ncol = 4)

i <- 1
for (a in seq(0,1, by=by)) {
  for (b in seq(0, (1-a), by = by)) {
    for (c in seq(0,(1-a-b), by = by)) {
      d <- 1-sum(c(a, b, c))
      if ( (a > (c-e)) && (b > (c-e)) && (b > (d-e)) ) {
        res[i,] <- c(a, b, c, d)
        i  <- i+1
      }
    }
  }
}


res_p8 <- round(res[1:(i-1),] * to_integer)

head(res_p8, 20)


p8 <- "
x1 +                   (1-x1)*x2*(1-x3)*(1/3) +                          (1-x1)*x2*x3*(1/4) 
(1-x1)*(1-x2)*(1-x3) + (1-x1)*x2*(1-x3)*(1/3) + (1-x1)*(1-x2)*x3*(1/2) + (1-x1)*x2*x3*(1/4) 
                       (1-x1)*x2*(1-x3)*(1/3) +                          (1-x1)*x2*x3*(1/4) 
                                                (1-x1)*(1-x2)*x3*(1/2) + (1-x1)*x2*x3*(1/4) 
"



check.mpt(textConnection(p8))

sfInit( parallel=TRUE, cpus=nCPU, type = "SOCK" )
deviances_p8 <- fit.stepwise(res_p8, p8, stepsize)
sfStop()

save(deviances_p8, file = "deviances_p8.rda")


## Pattern IX:
# A > B > D and A > C

res <- matrix(-1, nrow=1e07, ncol = 4)

i <- 1
for (a in seq(0,1, by=by)) {
  for (b in seq(0, (1-a), by = by)) {
    for (c in seq(0,(1-a-b), by = by)) {
      d <- 1-sum(c(a, b, c))
      if ( (a > (b-e)) && (b > (d-e)) && (a > (c-e)) ) {
        res[i,] <- c(a, b, c, d)
        i  <- i+1
      }
    }
  }
}

#head(res, 20)

#res[i-1,]

res_p9 <- round(res[1:(i-1),] * to_integer)

head(res_p7, 20)
tail(res_p7, 20)


p9 <- "
(1-x1)*(1-x2)*(1-x3) + x1*(1-x2)*(1-x3)*(1/2) + (1-x1)*x2*(1-x3)*(1/2) + x1*x2*(1-x3)*(1/3) + (1-x2)*x3*(1/3) + x2*x3*(1/4)
                       x1*(1-x2)*(1-x3)*(1/2) +                          x1*x2*(1-x3)*(1/3) + (1-x2)*x3*(1/3) + x2*x3*(1/4)
                                                (1-x1)*x2*(1-x3)*(1/2) + x1*x2*(1-x3)*(1/3) +                   x2*x3*(1/4)
                                                                                              (1-x2)*x3*(1/3) + x2*x3*(1/4)
"

check.mpt(textConnection(p9))

sfInit( parallel=TRUE, cpus=nCPU, type = "SOCK" )
deviances_p9 <- fit.stepwise(res_p9, p9, stepsize)
sfStop()

save(deviances_p9, file = "deviances_p9.rda")


## Pattern X:
# A > B, A > C, and A > D

res <- matrix(-1, nrow=1e07, ncol = 4)

i <- 1
for (a in seq(0,1, by=by)) {
  for (b in seq(0, (1-a), by = by)) {
    for (c in seq(0,(1-a-b), by = by)) {
      d <- 1-sum(c(a, b, c))
      if ( (a > (b-e)) && (a > (c-e)) && (a > (d-e)) ) {
        res[i,] <- c(a, b, c, d)
        i  <- i+1
      }
    }
  }
}


res_p10 <- round(res[1:(i-1),] * to_integer)


p10 <- "
(1-x1)*(1-x2)*(1-x3) + x1*(1-x2)*(1-x3)*(1/2) + (1-x1)*x2*(1-x3)*(1/2) + (1-x1)*(1-x2)*x3*(1/2) + x1*x2*(1-x3)*(1/3) + x1*(1-x2)*x3*(1/3) + (1-x1)*x2*x3*(1/3) + x1*x2*x3*(1/4)
                       x1*(1-x2)*(1-x3)*(1/2) +                                                   x1*x2*(1-x3)*(1/3) + x1*(1-x2)*x3*(1/3) +                    + x1*x2*x3*(1/4)
                                                (1-x1)*x2*(1-x3)*(1/2) +                          x1*x2*(1-x3)*(1/3) +                      (1-x1)*x2*x3*(1/3) + x1*x2*x3*(1/4)
                                                                         (1-x1)*(1-x2)*x3*(1/2) +                      x1*(1-x2)*x3*(1/3) + (1-x1)*x2*x3*(1/3) + x1*x2*x3*(1/4)
"

check.mpt(textConnection(p10))

sfInit( parallel=TRUE, cpus=nCPU, type = "SOCK" )
deviances_p10 <- fit.stepwise(res_p10, p10, stepsize)
sfStop()

save(deviances_p10, file = "deviances_p10.rda")


