
require(MPTinR)     # for fitting
require(snowfall)   # for multicore 
require(mail)       # for sending results to mail

# first generate all possible proportions (stepsize is by)
# then fit all possible porportions.
# then see if misfit exists.

by <- 0.005
to_integer <- 1000
by*to_integer
e <- by/100

n_optim <- 20
stepsize <- 1200

nCPU <- 12  # uses nCPU cores for multicore fitting

# results are automatically send to following mail address (cannot contain . or _, gmail.com works well)
mail_address <- "yourname@gmail.com"

fit.stepwise <- function(data, model, stepsize, check = "within", multicore = "individual", ncol = 4) {
  steps <- c(seq(1, nrow(data), by = stepsize), nrow(data))
  out <- matrix(-1, nrow=nrow(data), ncol=ncol)
  for (i in 2:length(steps)) {
    tmp_fit <- fit.model(data[steps[i-1]:steps[i],], textConnection(model), n.optim=n_optim, multicore=multicore, fit.aggregated=FALSE, use.gradient=FALSE, numDeriv=FALSE)
    out[steps[i-1]:steps[i],] <- abs(tmp_fit$data$observed$individual - tmp_fit$data$predicted$individual)
  }
  #browser()
  
  model_char <- as.character(match.call()[[3]])
  mis_in <- rowSums(out)  
    
  if (check == "within")   {
    text_to_mail <- paste(model_char, "\n", 
                          "max possible sets (indiv):", max(out),"\n", 
                          "deviation (r/c =", paste(which(out == max(out), arr.ind=TRUE), collapse = " "), "):", paste(out[which(out == max(out), arr.ind=TRUE)[,1],], collapse = " "),"\n", 
                          "observed  (r/c =", paste(which(out == max(out), arr.ind=TRUE), collapse= " "), "):", paste(data[which(out == max(out), arr.ind=TRUE)[,1],], collapse = " "), "\n", "\n",
                          "max possible sets (summed per row):", max(mis_in),"\n", 
                          "deviation (row =", which(mis_in == max(mis_in)), "):", paste(out[which(mis_in == max(mis_in)),], collapse=" "),"\n", 
                          "observed  (row =", which(mis_in == max(mis_in)), "):", paste(data[which(mis_in == max(mis_in)),], collapse=" "), "\n", "\n", "\n")
    
    cat(model_char, "\n")
    cat("max possible sets (indiv):", max(out),"\n")
    cat("deviation (r/c =", which(out == max(out), arr.ind=TRUE), "):", out[which(out == max(out), arr.ind=TRUE)[,1],],"\n")
    cat("observed  (r/c =", which(out == max(out), arr.ind=TRUE), "):", data[which(out == max(out), arr.ind=TRUE)[,1],], "\n", "\n")
    
    
    cat("max possible sets (summed per row):", max(mis_in),"\n")
    cat("deviation (row =", which(mis_in == max(mis_in)), "):", out[which(mis_in == max(mis_in)),],"\n")
    cat("observed  (row =", which(mis_in == max(mis_in)), "):", data[which(mis_in == max(mis_in)),], "\n", "\n", "\n")
    
  } else {
        text_to_mail <- paste(model_char, "\n", 
                          "min disallowed sets (summed per row):", min(mis_in),"\n",
                          "deviation (row =", which(mis_in == min(mis_in)), "):", paste(out[which(mis_in == min(mis_in)),], collapse=" "),"\n", 
                          "observed  (row =", which(mis_in == min(mis_in)), "):", paste(data[which(mis_in == min(mis_in)),], collapse= " "), "\n", "\n")
    
    cat(model_char, "\n")
    cat("min disallowed sets (summed per row):", min(mis_in),"\n")
    cat("deviation (row =", which(mis_in == min(mis_in)), "):", out[which(mis_in == min(mis_in)),],"\n")
    cat("observed  (row =", which(mis_in == min(mis_in)), "):", data[which(mis_in == min(mis_in)),], "\n")
  }
  
  sendmail(mail_address, paste("model", model_char), text_to_mail)
  

  
  out
  
}



## Binary inverted fork-like order constraint  (Tree 8)
## A > C and B > C

res_in <- matrix(-1, nrow=1e07, ncol = 3)
res_out <- matrix(-1, nrow=1e07, ncol = 3)

i_in <- 1
i_out <- 1
for (a in seq(0,1, by=by)) {
  for (b in seq(0, (1-a), by = by)) {
    c <- 1-sum(c(a, b))
    if ( (a > (c-e)) && (b > (c-e)) ) {
      res_in[i_in,] <- c(a, b, c)
      i_in  <- i_in+1
    } else {
      res_out[i_out,] <- c(a, b, c)
      i_out  <- i_out+1
    }
  }
}


data_in <- round(res_in[1:(i_in-1),] * to_integer)
data_out <- round(res_out[1:(i_out-1),] * to_integer)

i_in + i_out-2

head(data_in)
head(data_out)


tree_8 <- "
(1/3)*(1-x2)*x1     + (2/3)*(1/2)*x2 + 2*(2/3)*(1/2)*(1-x2)*x1
(1/3)*(1-x2)*(1-x1) + (2/3)*(1/2)*x2 + 2*(2/3)*(1/2)*(1-x2)*(1-x1)
(1/3)*x2
"


check.mpt(textConnection(tree_8))

sfInit( parallel=TRUE, cpus=nCPU, type = "SOCK" )
deviances_tree8_in <- fit.stepwise(data_in, tree_8, stepsize, ncol=3)
deviances_tree8_out <- fit.stepwise(data_out, tree_8, stepsize, check="out", ncol=3)
sfStop()

save(deviances_tree8_in, deviances_tree8_out, file = "results_tree_8.rda")



## Binary inverted fork-like order constraint with linear order below (Tree 9 [+ 8])
## A > C > D and B > C > D

res_in <- matrix(-1, nrow=1e07, ncol = 4)
res_out <- matrix(-1, nrow=1e07, ncol = 4)

i_in <- 1
i_out <- 1
for (a in seq(0,1, by=by)) {
  for (b in seq(0, (1-a), by = by)) {
    for (c in seq(0,(1-a-b), by = by)) {
      d <- 1-sum(c(a, b, c))
      if ( (a > (c-e)) && (c > (d-e)) && (b > (c-e)) ) {
        res_in[i_in,] <- c(a, b, c, d)
        i_in <- i_in+1
      } else {
        res_out[i_out,] <- c(a, b, c, d)
        i_out <- i_out+1
      }
    }
  }
}

i_in+i_out-2

data_in <- round(res_in[1:(i_in-1),] * to_integer)
data_out <- round(res_out[1:(i_out-1),] * to_integer)

dim(data_in)
dim(data_out)

tree_9 <- "
(1/4)*x0 + (1-x0)*((1/3)*(1-x2)*x1     + (2/3)*(1/2)*x2 + 2*(2/3)*(1/2)*(1-x2)*x1    )
(1/4)*x0 + (1-x0)*((1/3)*(1-x2)*(1-x1) + (2/3)*(1/2)*x2 + 2*(2/3)*(1/2)*(1-x2)*(1-x1))
(1/4)*x0 + (1-x0)*((1/3)*x2                                                          )
(1/4)*x0
"

check.mpt(textConnection(tree_9))


sfInit( parallel=TRUE, cpus=nCPU, type = "SOCK" )
deviances_tree9_in <- fit.stepwise(data_in, tree_9, stepsize)
deviances_tree9_out <- fit.stepwise(data_out, tree_9, stepsize, check="out")
sfStop()

save(deviances_tree9_in, deviances_tree9_out, file = "results_tree_9.rda")


## Combining forks: Pattern V (Trees 10 + 11)
## A > B > D and A > C > D

res_in <- matrix(-1, nrow=1e07, ncol = 4)
res_out <- matrix(-1, nrow=1e07, ncol = 4)

i_in <- 1
i_out <- 1
for (a in seq(0,1, by=by)) {
  for (b in seq(0, (1-a), by = by)) {
    for (c in seq(0,(1-a-b), by = by)) {
      d <- 1-sum(c(a, b, c))
      if ( (a > (b-e)) && (b > (d-e)) && (a > (c-e)) && (c > (d-e)) ) {
        res_in[i_in,] <- c(a, b, c, d)
        i_in <- i_in+1
      } else {
        res_out[i_out,] <- c(a, b, c, d)
        i_out <- i_out+1
      }
    }
  }
}

data_in <- round(res_in[1:(i_in-1),] * to_integer)
data_out <- round(res_out[1:(i_out-1),] * to_integer)


tree_10_11 <- "
(1/4) +          (3/4)*(1-x1)*((1/9)          + (8/9)*(1/2)*(1-x2)*(1/2)*(1-x3) + (8/9)*(1/2)*(1-x3)*(1/2)*(1-x2) + (8/9)*(1/2)*(1-x2)*(1/2) + (8/9)*(1/2)*(1-x3)*(1/2))
(3/4)*(1/3)*x1 + (3/4)*(1-x1)*((8/9)*(1/2)*x2 +                                   (8/9)*(1/2)*(1-x3)*(1/2)*x2                                                          )
(3/4)*(1/3)*x1 + (3/4)*(1-x1)*((8/9)*(1/2)*x3 + (8/9)*(1/2)*(1-x2)*(1/2)*x3                                                                                            )
(3/4)*(1/3)*x1 
"


check.mpt(textConnection(tree_10_11))

sfInit( parallel=TRUE, cpus=nCPU, type = "SOCK" )
deviances_tree10_11_in <- fit.stepwise(data_in, tree_10_11, stepsize)
deviances_tree10_11_out <- fit.stepwise(data_out, tree_10_11, stepsize, check="out")
sfStop()

save(deviances_tree10_11_in, deviances_tree10_11_out, file = "results_tree_10_11.rda")



## Binary fork plus linear order above (Appendix B)
## F > A > B and F > A > C

res_in <- matrix(-1, nrow=1e07, ncol = 4)
res_out <- matrix(-1, nrow=1e07, ncol = 4)

i_in <- 1
i_out <- 1
for (a in seq(0,1, by=by)) {
  for (b in seq(0, (1-a), by = by)) {
    for (c in seq(0,(1-a-b), by = by)) {
      f <- 1-sum(c(a, b, c))
      if ( (f > (a-e)) && (a > (b-e)) && (a > (c-e)) ) {
        res_in[i_in,] <- c(f, a, b, c)
        i_in <- i_in+1
      } else {
        res_out[i_out,] <- c(f, a, b, c)
        i_out <- i_out+1
      }
    }
  }
}

data_in <- round(res_in[1:(i_in-1),] * to_integer)
data_out <- round(res_out[1:(i_out-1),] * to_integer)


appendix_b <- "
(1/4)+(3/4)*x0     +                               (3/4)*(1-x0)*(2/3)*(1/2)*(1-x1)*(2/3)*(1/2) + (3/4)*(1-x0)*(2/3)*(1/2)*(1-x2)*(2/3)*(1/2) + (3/4)*(1-x0)*(2/3)*(1/2)*(1-x1)*(1/3)*(1-x2)*((1/2) + (1/2)*x0) + (3/4)*(1-x0)*(2/3)*(1/2)*(1-x2)*(1/3)*(1-x1)*((1/2) + (1/2)*x0)  
(3/4)*(1-x0)*(1/3) +                               (3/4)*(1-x0)*(2/3)*(1/2)*(1-x1)*(2/3)*(1/2) + (3/4)*(1-x0)*(2/3)*(1/2)*(1-x2)*(2/3)*(1/2) + (3/4)*(1-x0)*(2/3)*(1/2)*(1-x1)*(1/3)*(1-x2)*(1/2)*(1-x0)       + (3/4)*(1-x0)*(2/3)*(1/2)*(1-x2)*(1/3)*(1-x1)*(1/2)*(1-x0)        
                     (3/4)*(1-x0)*(2/3)*(1/2)*x1 + (3/4)*(1-x0)*(2/3)*(1/2)*(1-x2)*(1/3)*x1     
                     (3/4)*(1-x0)*(2/3)*(1/2)*x2 +                                               (3/4)*(1-x0)*(2/3)*(1/2)*(1-x1)*(1/3)*x2    
"


check.mpt(textConnection(appendix_b))


sfInit( parallel=TRUE, cpus=nCPU, type = "SOCK" )
deviances_appendix_b_in <- fit.stepwise(data_in, appendix_b, stepsize)
deviances_appendix_b_out <- fit.stepwise(data_out, appendix_b, stepsize, check="out")
sfStop()

save(deviances_appendix_b_in, deviances_appendix_b_out, file = "results_appendix_b_in.rda")



